@@ -268,6 +268,8 @@ static int fork_qemu(const char* hda, const char* hdb, const struct pbuilderconf
       /* this is the child process */
       const char* qemu = qemu_arch_qemu(pc->arch);
       const char* machine = qemu_arch_qemumachine(pc->arch);
+      char* hda_command;
+      char* hdb_command;
       char* append_command;
       const char* kernel_image = pc->kernel_image;
       const char* initrd = pc->initrd;
@@ -284,6 +286,12 @@ static int fork_qemu(const char* hda, const char* hdb, const struct pbuilderconf
 
       asprintf(&mem, "%i", pc->memory_megs);
 
+      asprintf(&hda_command, "file=%s,index=0,media=disk,cache=writeback",
+	       strdupa(hda));
+
+      asprintf(&hdb_command, "file=%s,index=1,media=disk,cache=writeback",
+	       strdupa(hdb));
+
       asprintf(&append_command,
 	       "root=/dev/%sa quiet init=/pbuilder-run console=%s",
 	       qemu_arch_diskdevice(pc),
@@ -307,14 +315,18 @@ static int fork_qemu(const char* hda, const char* hdb, const struct pbuilderconf
 	  argv[argc++]="-initrd";
 	  argv[argc++]=strdupa(initrd);
 	}
-      argv[argc++]="-hda";
-      argv[argc++]=strdupa(hda);
-      argv[argc++]="-hdb";
-      argv[argc++]=strdupa(hdb);
+      argv[argc++]="-drive";
+      argv[argc++]=hda_command;
+      argv[argc++]="-drive";
+      argv[argc++]=hdb_command;
       argv[argc++]="-append";
       argv[argc++]=append_command;
       argv[argc++]="-serial";
       argv[argc++]="stdio";
+      argv[argc++]="-net";
+      argv[argc++]="user";
+      argv[argc++]="-net";
+      argv[argc++]="nic";
       argv[argc]=NULL;
       assert(argc < MAX_ARGS);
 
@@ -471,13 +483,13 @@ static int run_second_stage_script
       system(hostcommand1);
     }
 
-  loop_umount(workblockdevicepath);
+  loop_umount(pc->buildplace);
 
   asprintf(&cowdevpath, "%s.cowdev", pc->buildplace);
   ret=forkexeclp("qemu-img", "qemu-img",
 		 "create",
 		 "-f",
-		 "qcow",
+		 "qcow2",
 		 "-b",
 		 pc->basepath,
 		 cowdevpath,
@@ -504,7 +516,7 @@ static int run_second_stage_script
       printf("running host command: %s\n", hostcommand2);
       system(hostcommand2);
     }
-  loop_umount(workblockdevicepath);
+  loop_umount(pc->buildplace);
   rmdir(pc->buildplace);
   printf(" -> clean up COW device files\n");
   unlink(workblockdevicepath);
@@ -722,7 +734,7 @@ int cpbuilder_create(const struct pbuilderconfig* pc)
 	  "$PBUILDER_MOUNTPOINT/run-copyfiles\n"
 	  "hostname pbuilder-$(cat /etc/hostname)\n"
 	  //TODO: installaptlines
-	  //"echo 'deb http://192.168.1.26/debian/ sid main ' > /etc/apt/sources.list\n"
+	  "echo '%s' > /etc/apt/sources.list.d/other.list\n"
 	  //TODO: run G hook
 	  "apt-get update || exit_from_qemu 1\n"
 	  //TODO: "dpkg --purge $REMOVEPACKAGES\n"
@@ -739,7 +751,7 @@ int cpbuilder_create(const struct pbuilderconfig* pc)
 	  "bash\n",
 	  timestring,
 	  timestring,
-	  t=sanitize_mirror(pc->mirror), pc->distribution, pc->components);
+	  t=sanitize_mirror(pc->mirror), pc->distribution, pc->components, pc->othermirror);
   fclose(f);
   free(t);
 
@@ -755,7 +767,7 @@ int cpbuilder_create(const struct pbuilderconfig* pc)
   /* TODO: hooks probably need copying here. */
   /* TODO: recover aptcache */
 
-  loop_umount(workblockdevicepath);
+  loop_umount(pc->buildplace);
   rmdir(pc->buildplace);
 
   // this will have wrong time. how to workaround?
@@ -786,8 +798,18 @@ int cpbuilder_build(const struct pbuilderconfig* pc, const char* dscfile)
   char* hoststr=NULL;
   char* hoststr2=NULL;
   char* commandline=NULL;
+  const char* buildopt=NULL;
+  const char* debbuildopts=NULL;
+  char* debbuildopts_work=NULL;
 
-  const char* buildopt="--binary-all"; /* TODO: add --binary-arch option */
+  if (pc->binary_arch) {
+    asprintf(&debbuildopts_work, "%s -B", pc->debbuildopts);
+    debbuildopts=debbuildopts_work;
+    buildopt="--binary-arch";
+  } else {
+    debbuildopts=pc->debbuildopts;
+    buildopt="--binary-all";
+  }
 
   hoststr=copy_dscfile(dscfile, pc->buildplace);
 
@@ -800,7 +822,7 @@ int cpbuilder_build(const struct pbuilderconfig* pc, const char* dscfile)
 	   "cd $PBUILDER_MOUNTPOINT/*-*/; dpkg-buildpackage -us -uc %s\n",
 	   buildopt,
 	   dscfile,
-	   pc->debbuildopts);
+	   debbuildopts);
 
   /* Obscure assumption!: assume _ is significant for package name and
      no other file will have _. */
@@ -815,6 +837,7 @@ int cpbuilder_build(const struct pbuilderconfig* pc, const char* dscfile)
      hoststr,
      hoststr2);
 
+  if(debbuildopts_work) free(debbuildopts_work);
   if(hoststr2) free(hoststr2);
   if(hoststr) free(hoststr);
   if(commandline) free(commandline);
@@ -858,7 +881,7 @@ int cpbuilder_execute(const struct pbuilderconfig* pc, char** av)
  */
 int cpbuilder_update(const struct pbuilderconfig* pc)
 {
-  /* TODO: --override-config support.
+  /* TODO: --override-config support, --othermirror support etc.
      There is no way to change distribution in this code-path...
    */
   return run_second_stage_script
